html,body {
     height:100%;
     width:100%;
     margin:0;
}
 body {
     display:flex;
}
 .logo {
     width: 75px;
     height: 75px;
     float: left;
     margin: 20px 20px 0px 20px;
     -webkit-animation:spin 8s linear infinite;
}
 @-webkit-keyframes spin {
     100% {
         -webkit-transform: rotate(360deg);
    }
}
 .header {
     display: inline
}
 .form {
     width: 480px;
     height: 170px;
     background: -webkit-linear-gradient(bottom, #ddd, #fff);
     border: 1px solid #999;
     border-radius: 12px;
     color: #46a;
     font-family: 'Lucida Sans Unicode', 'Lucida Grande', sans-serif;
     font-size: 14px;
     font-style: italic;
     font-weight: bold;
     margin: auto;
     padding: 10px;
     position: relative;
     line-height: 26px;
     text-decoration: none;
     -webkit-box-shadow: 0px 0px 5px #444;
}
 h1 {
     padding-left:40px;
     color: #46a2da;
}
 h2 {
     color: #80c342;
     font-size: 13px;
     margin-top: -20px;
}
 span {
     margin-left: 20px;
}
 .button{
     display: inline-block;
     background: #46a2da;
     width: 100px;
     height: 30px;
     padding: 0px;
     text-align: center;
     font-weight: bold;
     color: #ffffff;
     text-decoration: none;
     border: 1px solid #999;
     margin-left: 190px;
}
 .button:hover {
    background-color: #46a200
}
 .button:active {
     background-color: #3e8e41;
}
