.logo {
 width: 50px;
 height: 50px;
 float: left;
 margin: 20px 20px 0px 20px;
 -webkit-animation:spin 8s linear infinite;
}
@-webkit-keyframes spin { 100% { -webkit-transform: rotate(360deg); } }

.header {
 display: inline
}

.form {
  width: 450px;
  height: 600px;
  background: -webkit-linear-gradient(bottom, #ddd, #fff);
  border: 1px solid #999;
  border-radius: 12px;
  color: #46a;
  font-family: 'Lucida Sans Unicode', 'Lucida Grande', sans-serif;
  font-size: 14px;
  font-style: italic;
  font-weight: bold;
  margin: auto;
  padding: 10px;
  position: relative;
  line-height: 26px;
  text-decoration: none;
  -webkit-box-shadow: 0px 0px 5px #444;
}

h1 {
 padding-left:40px;
 color: #46a2da;
}

h2 {
 color: #80c342;
 font-size: 13px;
 margin-top: -20px;
}

span {
  margin-left: 20px;
}

input {
  width: 400px;
  display: block;
  border: 1px solid #999;
  height: 25px;
  margin-left: 20px;
  margin-bottom: 10px;
  padding-left: 10px;
  -webkit-box-shadow: 0px 0px 5px rgba(0, 0, 0, 0.3);
}

textarea {
  width: 400px;
  max-width: 400px;
  height: 180px;
  max-height: 400px;
  display: block;
  margin-left: 20px;
  padding-left: 10px;
  padding-right: 10px;
  -webkit-box-shadow: 0px 0px 5px rgba(0, 0, 0, 0.3);
}

input:focus, textarea:focus {
 outline:none;
 border: 1px solid #46a2da;
}

input[type=submit] {
  width: 100px;
  left: 170px;
  bottom: 10px;
  background: #46a2da;
  color: #fff;
  height: 30px;
  position: absolute;
  border-radius: 14px;
}

input[type=submit]:hover {
  background: #80c342;
}

