/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** BSD License Usage
** Alternatively, you may use this file under the terms of the BSD license
** as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of The Qt Company Ltd nor the names of its
**     contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "mesh.h"
#include <QtConcurrentRun>
#include <QFile>

void Mesh::load(const QString &fn)
{
    reset();
    m_maybeRunning = true;
    m_future = QtConcurrent::run([fn]() {
        MeshData md;
        QFile f(fn);
        if (!f.open(QIODevice::ReadOnly)) {
            qWarning("Failed to open %s", qPrintable(fn));
            return md;
        }
        QByteArray buf = f.readAll();
        const char *p = buf.constData();
        quint32 format;
        memcpy(&format, p, 4);
        if (format != 1) {
            qWarning("Invalid format in %s", qPrintable(fn));
            return md;
        }
        int ofs = 4;
        memcpy(&md.vertexCount, p + ofs, 4);
        ofs += 4;
        memcpy(md.aabb, p + ofs, 6 * 4);
        ofs += 6 * 4;
        const int byteCount = md.vertexCount * 8 * 4;
        md.geom.resize(byteCount);
        memcpy(md.geom.data(), p + ofs, byteCount);
        return md;
    });
}

MeshData *Mesh::data()
{
    if (m_maybeRunning && !m_data.isValid())
        m_data = m_future.result();

    return &m_data;
}

void Mesh::reset()
{
    *data() = MeshData();
    m_maybeRunning = false;
}
