.div {
    padding:8px 4px;
    border: 5px solid #188BD0;
    width: 280px;
    font-family: sans-serif;
    font-size:10pt;
}
.link {
    text-decoration: none;
    color: #888888;
}
.button {
    background: -webkit-linear-gradient(top,#25A6E2 0%,#188BD0 100%);
    padding:8px 13px;
    color:#fff;
    font-family: sans-serif;
    font-size:17px;
    -webkit-border-radius:5px;
    border:1px solid #1A87FF;
    width: 300px;
}
.button:focus {
    outline: none;
}
.button:active {
    background: -webkit-linear-gradient(top,#25A6E2 0%,#188BD0 70%);
}
.input {
    padding:8px 4px;
    border: 5px solid #188BD0;
    width: 280px;
    font-family: sans-serif;
    font-size:10pt;
}
.input:focus {
    outline: none;
}
